@@ -1,5 +1,6 @@
 # -*- coding:utf-8 -*-
 
+import random
 from collections import defaultdict
 
 from grapheekdb.lib.undef import UNDEFINED
@@ -10,6 +11,7 @@ from grapheekdb.lib.exceptions import GrapheekDataException, GrapheekDataPrepara
 from grapheekdb.lib.exceptions import GrapheekIndexCreationFailedException, GrapheekIndexRemovalFailedException, GrapheekNoSuchTraversalException
 from grapheekdb.lib.exceptions import GrapheekMissingKeyException, GrapheekInvalidDataTypeException
 from grapheekdb.lib.exceptions import GrapheekUnknownScriptException, GrapheekUnknownAlias
+from grapheekdb.lib.exceptions import GrapheekUnknownMethod
 from grapheekdb.lib.expressions import eval_expr
 
 from grapheekdb.backends.data.keys import build_key, PREPARED
@@ -25,10 +27,16 @@ from grapheekdb.backends.data.operations import Addition, Removal
 from grapheekdb.backends.data.optimizer import Optimizer
 
 
-# misc iterator :
+# misc iterators :
 def _jump_iterator(_graph, _kind, _entity_id_iterator, _traversal, _neighbors_cache, **filters):
     for entity_id in _entity_id_iterator:
-        for neighbor_id in iter(_graph._neighbors(_kind, entity_id, _traversal, _neighbors_cache, **filters)):
+        for neighbor_id in iter(_graph._neighbors(_kind, entity_id, _traversal, _neighbors_cache, False, **filters)):
+            yield neighbor_id
+
+
+def _random_iterator(_graph, _kind, _entity_id_iterator, _traversal, _neighbors_cache, **filters):
+    for entity_id in _entity_id_iterator:
+        for neighbor_id in iter(_graph._neighbors(_kind, entity_id, _traversal, _neighbors_cache, True, **filters)):
             yield neighbor_id
 
 
@@ -261,7 +269,8 @@ class EntityIterator(object):
     def __iter__(self):
         return self
 
-    def _jump(self, _kind, _traversal, *args, **filters):
+    def _jump(self, _kind, _traversal, _random, *args, **filters):
+        jumping_iterator = _random_iterator if _random else _jump_iterator
         # TODO : Refactor this function : it can certainly be rewritten in a far easier way
         count = check_and_get_count(args)
         current_iterator = self
@@ -275,7 +284,7 @@ class EntityIterator(object):
                 self._graph,
                 current_kind,
                 filters,
-                _jump_iterator(
+                jumping_iterator(
                     self._graph,
                     parent_kind,
                     current_iterator._iterate(),
@@ -304,13 +313,13 @@ class EntityIterator(object):
         raise StopIteration
 
     def inV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, IN_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, IN_VERTICES_SUFFIX, False, *args, **filters)
 
     def outV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, OUT_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, OUT_VERTICES_SUFFIX, False, *args, **filters)
 
     def bothV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, BOTH_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, BOTH_VERTICES_SUFFIX, False, *args, **filters)
 
     def in_(self, *args, **filters):
         assert(self._src_kind == KIND_VERTEX)
@@ -375,17 +384,41 @@ class EntityIterator(object):
     def inE(self, *args, **filters):
         if self._src_kind == KIND_EDGE:
             raise GrapheekNoSuchTraversalException
-        return self._jump(KIND_EDGE, IN_EDGES_SUFFIX, **filters)  # no *args for inE ()
+        return self._jump(KIND_EDGE, IN_EDGES_SUFFIX, False, **filters)  # no *args for inE ()
 
     def outE(self, *args, **filters):
         if self._src_kind == KIND_EDGE:
             raise GrapheekNoSuchTraversalException
-        return self._jump(KIND_EDGE, OUT_EDGES_SUFFIX, **filters)
+        return self._jump(KIND_EDGE, OUT_EDGES_SUFFIX, False, **filters)
 
     def bothE(self, *args, **filters):
         if self._src_kind == KIND_EDGE:
             raise GrapheekNoSuchTraversalException
-        return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, **filters)
+        return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, False, **filters)
+
+    def random(self, _method, *args, **filters):
+        if self._src_kind == KIND_VERTEX:
+            try:
+                kind, traversal = {
+                    'inV': (KIND_VERTEX, IN_VERTICES_SUFFIX),
+                    'outV': (KIND_VERTEX, OUT_VERTICES_SUFFIX),
+                    'bothV': (KIND_VERTEX, BOTH_VERTICES_SUFFIX),
+                    'inE': (KIND_EDGE, IN_EDGES_SUFFIX),
+                    'outE': (KIND_EDGE, OUT_EDGES_SUFFIX),
+                    'bothE': (KIND_EDGE, BOTH_EDGES_SUFFIX)
+                }[_method]
+            except KeyError:
+                raise GrapheekUnknownMethod(_method)
+        elif self._src_kind == KIND_EDGE:
+            try:
+                kind, traversal = {
+                    'inV': (KIND_VERTEX, IN_VERTICES_SUFFIX),
+                    'outV': (KIND_VERTEX, OUT_VERTICES_SUFFIX),
+                    'bothV': (KIND_VERTEX, BOTH_VERTICES_SUFFIX)
+                }[_method]
+            except KeyError:
+                raise GrapheekUnknownMethod(_method)
+        return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, True, **filters)
 
     def dedup(self, *aliases):
         def dedup_iterator(entity_iterator, aliases):
@@ -535,10 +568,11 @@ class Entity(object):
         else:
             self._graph._update_data(self._kind, self._entity_id, attr, value)
 
-    def _neighbors(self, _traversal, **filters):
-        return self._graph._neighbors(self._kind, self._entity_id, _traversal, **filters)
+    def _neighbors(self, _traversal, _random, **filters):
+        return self._graph._neighbors(self._kind, self._entity_id, _traversal, _cache=None, _random=_random, **filters)
 
-    def _jump(self, _tgt_kind, _traversal, *args, **filters):
+    def _jump(self, _tgt_kind, _traversal, _random, *args, **filters):
+        jumping_iterator = _random_iterator if _random else _jump_iterator
         # TODO : Factor code with EntityIterator._jump
         count_args = len(args)
         assert(count_args < 2)
@@ -546,13 +580,13 @@ class Entity(object):
         count = 1
         if count_args == 1:
             count = args[0]
-        current_iterator = self._neighbors(_traversal, **filters)
+        current_iterator = self._neighbors(_traversal, _random, **filters)
         parent_kind = self._kind
         current_kind = _tgt_kind
         current_parent = None
         # (potentially) recursive jumping
         for i in range(count):
-            IT = current_iterator if i == 0 else _jump_iterator(self._graph, parent_kind, current_iterator._iterate(), _traversal, {}, **filters)
+            IT = current_iterator if i == 0 else jumping_iterator(self._graph, parent_kind, current_iterator._iterate(), _traversal, {}, **filters)
             current_iterator = EntityIterator(
                 self._graph,
                 current_kind,
@@ -572,13 +606,24 @@ class Entity(object):
             return default
 
     def inV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, IN_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, IN_VERTICES_SUFFIX, False, *args, **filters)
 
     def outV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, OUT_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, OUT_VERTICES_SUFFIX, False, *args, **filters)
 
     def bothV(self, *args, **filters):
-        return self._jump(KIND_VERTEX, BOTH_VERTICES_SUFFIX, *args, **filters)
+        return self._jump(KIND_VERTEX, BOTH_VERTICES_SUFFIX, False, *args, **filters)
+
+    def random(self, _method, **filters):
+        try:
+            kind, traversal = {
+                'inV': (KIND_VERTEX, IN_VERTICES_SUFFIX),
+                'outV': (KIND_VERTEX, OUT_VERTICES_SUFFIX),
+                'bothV': (KIND_VERTEX, BOTH_VERTICES_SUFFIX)
+            }[_method]
+        except KeyError:
+            raise GrapheekUnknownMethod(_method)
+        return self._jump(kind, traversal, True, **filters)
 
     def data(self):
         return self._graph._get_data(None, self._kind, self._entity_id)
@@ -743,13 +788,27 @@ class Node(Entity):
         return entity_iterator.both_(*args, **filters)
 
     def inE(self, **filters):
-        return self._jump(KIND_EDGE, IN_EDGES_SUFFIX, **filters)
+        return self._jump(KIND_EDGE, IN_EDGES_SUFFIX, False, **filters)
 
     def outE(self, **filters):
-        return self._jump(KIND_EDGE, OUT_EDGES_SUFFIX, **filters)
+        return self._jump(KIND_EDGE, OUT_EDGES_SUFFIX, False, **filters)
 
     def bothE(self, **filters):
-        return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, **filters)
+        return self._jump(KIND_EDGE, BOTH_EDGES_SUFFIX, False, **filters)
+
+    def random(self, _method, **filters):
+        try:
+            kind, traversal = {
+                'inV': (KIND_VERTEX, IN_VERTICES_SUFFIX),
+                'outV': (KIND_VERTEX, OUT_VERTICES_SUFFIX),
+                'bothV': (KIND_VERTEX, BOTH_VERTICES_SUFFIX),
+                'inE': (KIND_EDGE, IN_EDGES_SUFFIX),
+                'outE': (KIND_EDGE, OUT_EDGES_SUFFIX),
+                'bothE': (KIND_EDGE, BOTH_EDGES_SUFFIX)
+            }[_method]
+        except KeyError:
+            raise GrapheekUnknownMethod(_method)
+        return self._jump(kind, traversal, True, **filters)
 
     def remove(self):
         self._graph._remove_node(self._entity_id)
@@ -1244,7 +1303,7 @@ class BaseGraph(GraphReadWrite, GraphNx):
 
     # Traversals given entity ids :
 
-    def _neighbors(self, _kind, _entity_id, _traversal, _cache=None, **filters):
+    def _neighbors(self, _kind, _entity_id, _traversal, _cache=None, _random=False, **filters):
         """
         _neighbors is using a cache (transmitted by iterator) to speed up lookup
         This is done to asymptotically improve performance at the expense of local memory
@@ -1271,8 +1330,11 @@ class BaseGraph(GraphReadWrite, GraphNx):
                 _cache[cache_key] = neighbour_ids
         # Special case when filters is empty or kind is edge (no need to look in index in the case of edge)
         if (_kind == KIND_EDGE) or not filters:
-            for entity_id in iter(neighbour_ids):
-                yield entity_id
+            if not(_random):
+                for entity_id in iter(neighbour_ids):
+                    yield entity_id
+            else:
+                yield random.choice(neighbour_ids)
         else:
             # Preparing metrics that will allow to choose between sequential access and index access :
             iterator = None
@@ -1289,12 +1351,18 @@ class BaseGraph(GraphReadWrite, GraphNx):
                 # which can cost a lot (<index_count> disk access for persistent backends)
                 iterator = self._optimizer.index_or_seq_scan_iterator(_kind, seq_count * 2, **filters)
             if iterator is None:
-                for entity_id in iter(neighbour_ids):
-                    yield entity_id
+                if not(_random):
+                    for entity_id in iter(neighbour_ids):
+                        yield entity_id
+                else:
+                    yield random.choice(neighbour_ids)
             else:
                 indexed_ids = set(iterator)
-                for entity_id in indexed_ids.intersection(neighbour_ids):
-                    yield entity_id
+                if not(_random):
+                    for entity_id in indexed_ids.intersection(neighbour_ids):
+                        yield entity_id
+                else:  # pragma : no cover
+                    yield random.choice(indexed_ids.intersection(neighbour_ids))
 
     # Don't override next methods : it's the public interface
     # (or some "private" helper methods for public methods)
